@@ -16,16 +16,17 @@
 
 #include <t3widget/widget.h>
 
-using namespace t3_widget;
+using namespace t3widget;
 
 class file_autocompleter_t : public autocompleter_t {
  private:
-  string_list_t *current_list = nullptr;
-  int completion_start = 0;
+  std::unique_ptr<string_list_t> current_list;
+  text_pos_t completion_start = 0;
 
  public:
   file_autocompleter_t() = default;
-  string_list_base_t *build_autocomplete_list(const text_buffer_t *text, int *position) override;
+  string_list_base_t *build_autocomplete_list(const text_buffer_t *text,
+                                              text_pos_t *position) override;
   void autocomplete(text_buffer_t *text, size_t idx) override;
 };
 
