@@ -14,13 +14,8 @@
 #include <cstring>
 #include <fcntl.h>
 #include <unistd.h>
-#ifdef HAS_LIBATTR
-#include <attr/libattr.h>
-#endif
-#ifdef HAS_LIBACL
-#include <acl/libacl.h>
-#endif
 
+#include "tilde/copy_file.h"
 #include "tilde/filebuffer.h"
 #include "tilde/fileline.h"
 #include "tilde/filestate.h"
@@ -32,7 +27,7 @@
 
 file_buffer_t::file_buffer_t(string_view _name, string_view _encoding)
     : text_buffer_t(new file_line_factory_t(this)),
-      view_parameters(new edit_window_t::view_parameters_t()),
+      behavior_parameters(new edit_window_t::behavior_parameters_t()),
       has_window(false),
       highlight_valid(0),
       highlight_info(nullptr),
@@ -56,12 +51,12 @@ file_buffer_t::file_buffer_t(string_view _name, string_view _encoding)
 
   connect_rewrap_required(bind_front(&file_buffer_t::invalidate_highlight, this));
 
-  view_parameters->set_tabsize(option.tabsize);
-  view_parameters->set_wrap(option.wrap ? wrap_type_t::WORD : wrap_type_t::NONE);
-  view_parameters->set_auto_indent(option.auto_indent);
-  view_parameters->set_tab_spaces(option.tab_spaces);
-  view_parameters->set_indent_aware_home(option.indent_aware_home);
-  view_parameters->set_show_tabs(option.show_tabs);
+  behavior_parameters->set_tabsize(option.tabsize);
+  behavior_parameters->set_wrap(option.wrap ? wrap_type_t::WORD : wrap_type_t::NONE);
+  behavior_parameters->set_auto_indent(option.auto_indent);
+  behavior_parameters->set_tab_spaces(option.tab_spaces);
+  behavior_parameters->set_indent_aware_home(option.indent_aware_home);
+  behavior_parameters->set_show_tabs(option.show_tabs);
   open_files.push_back(this);
 }
 
@@ -163,6 +158,7 @@ rw_result_t file_buffer_t::load(load_process_t *state) {
         }
         set_cursor({0, 0});
       } catch (rw_result_t &result) {
+        state->buffer_used = false;
         return result;
       }
       break;
@@ -201,7 +197,14 @@ rw_result_t file_buffer_t::load(load_process_t *state) {
   }
   if (success) {
     highlight = t3_highlight_load(lang.lang_file, map_highlight, nullptr,
-                                  T3_HIGHLIGHT_UTF8 | T3_HIGHLIGHT_USE_PATH, nullptr);
+                                  T3_HIGHLIGHT_UTF8 | T3_HIGHLIGHT_USE_PATH
+/* If T3_HIGHLIGHT_USE_SCOPE is not available, all the other code is still compatible, so we simply
+   omit the flag here. */
+#ifdef T3_HIGHLIGHT_USE_SCOPE
+                                      | T3_HIGHLIGHT_USE_SCOPE
+#endif
+                                  ,
+                                  nullptr);
     set_highlight(highlight);
     std::map<std::string, std::string>::iterator iter = option.line_comment_map.find(lang.name);
     if (iter != option.line_comment_map.end()) {
@@ -221,10 +224,55 @@ rw_result_t file_buffer_t::save(save_as_process_t *state) {
   }
 
   switch (state->state) {
-    case save_as_process_t::INITIAL:
+    case save_as_process_t::INITIAL: {
+      if (strip_spaces.is_valid() ? strip_spaces.value() : option.strip_spaces) {
+        do_strip_spaces();
+      }
+
+      transcript_error_t error;
+      if (!state->encoding.empty()) {
+        if ((state->conversion_handle = transcript_open_converter(
+                 state->encoding.c_str(), TRANSCRIPT_UTF8, 0, &error)) == nullptr) {
+          return rw_result_t(rw_result_t::CONVERSION_OPEN_ERROR);
+        }
+        encoding = state->encoding;
+      } else if (encoding != "UTF-8") {
+        if ((state->conversion_handle = transcript_open_converter(encoding.c_str(), TRANSCRIPT_UTF8,
+                                                                  0, &error)) == nullptr) {
+          return rw_result_t(rw_result_t::CONVERSION_OPEN_ERROR);
+        }
+      } else {
+        state->conversion_handle = nullptr;
+      }
+      state->wrapper = t3widget::make_unique<file_write_wrapper_t>(-1, state->conversion_handle);
+      state->i = 0;
+      state->state = save_as_process_t::OPEN_FILE;
+    }
+      // FALLTHROUGH
+    case save_as_process_t::OPEN_FILE: {
+      if (state->conversion_handle) {
+        transcript_from_unicode_reset(state->conversion_handle);
+      }
+      try {
+        for (; state->i < size(); state->i++) {
+          if (state->i != 0) {
+            state->wrapper->write("\n", 1);
+          }
+          const std::string &data = get_line_data(state->i).get_data();
+          /* At this point the wrapper is initialized with -1 as fd, thus it is not writing
+             anything. However, it will catch conversion errors, and this will result in asking
+             the user what to do. */
+          state->wrapper->write(data.data(), data.size());
+        }
+      } catch (rw_result_t error) {
+        return error;
+      }
+      state->computed_length = state->wrapper->written_size();
+
       if (state->name.empty()) {
         if (name.empty()) {
-          PANIC();
+          // FIXME: this should return some result instead of crashing!
+          return rw_result_t(rw_result_t::INTERNAL_ERROR);
         }
         state->save_name = name.c_str();
       } else {
@@ -234,106 +282,131 @@ rw_result_t file_buffer_t::save(save_as_process_t *state) {
       state->real_name = canonicalize_path(state->save_name);
       if (state->real_name.empty()) {
         if (errno != ENOENT) {
-          return rw_result_t(rw_result_t::ERRNO_ERROR, errno);
+          return rw_result_t(rw_result_t::ERRNO_ERROR_FILE_UNTOUCHED, errno);
         }
         state->real_name = state->save_name;
       }
 
-      /* FIXME: to avoid race conditions, it is probably better to try open with O_CREAT|O_EXCL
-         However, this may cause issues with NFS, which is known to have isues with this. */
-      if (stat(state->real_name.c_str(), &state->file_info) < 0) {
-        if (errno == ENOENT) {
-          if ((state->fd = creat(state->real_name.c_str(), CREATE_MODE)) < 0) {
-            return rw_result_t(rw_result_t::ERRNO_ERROR, errno);
+      /* This attempts to avoid race conditions, by trying to open with O_CREAT|O_EXCL. This is
+         known to have issues on NFSv3, but it's the best we can do. */
+      if ((state->fd = open(state->real_name.c_str(), O_CREAT | O_EXCL | O_RDWR, 0666)) < 0) {
+        struct stat file_info;
+        if ((state->fd = open(state->real_name.c_str(), O_RDWR)) >= 0) {
+          state->state = save_as_process_t::CREATE_BACKUP;
+          if (!state->name.empty()) {
+            return rw_result_t(rw_result_t::FILE_EXISTS);
+          }
+        } else if ((state->readonly_fd = open(state->real_name.c_str(), O_RDONLY)) >= 0) {
+          if (fstat(state->readonly_fd, &file_info) == 0 && file_info.st_uid == geteuid() &&
+              (file_info.st_mode & S_IRUSR) && !(file_info.st_mode & S_IWUSR)) {
+            state->state = save_as_process_t::CHANGE_MODE;
+            state->original_mode = file_info.st_mode & 07777;
+            state->readonly_dev = file_info.st_dev;
+            state->readonly_ino = file_info.st_ino;
+            return rw_result_t(rw_result_t::READ_ONLY_FILE);
+          } else {
+            close(state->readonly_fd);
+            state->readonly_fd = -1;
+            return rw_result_t(rw_result_t::ERRNO_ERROR_FILE_UNTOUCHED, EACCES);
           }
         } else {
-          return rw_result_t(rw_result_t::ERRNO_ERROR, errno);
+          return rw_result_t(rw_result_t::ERRNO_ERROR_FILE_UNTOUCHED, errno);
+        }
+      }
+    }
+      if (0) {
+          /* The code here is only reachable through the case label. This is an ugly hack to prevent
+             executing this code in the normal path, but allow continuing execution after this block
+             once the file mode is changed. */
+        case save_as_process_t::CHANGE_MODE:
+          if (fchmod(state->readonly_fd, state->original_mode.value() | S_IWUSR)) {
+            return rw_result_t(rw_result_t::ERRNO_ERROR_FILE_UNTOUCHED);
+          }
+          if ((state->fd = open(state->real_name.c_str(), O_RDWR)) < 0) {
+            int saved_errno = errno;
+            fchmod(state->readonly_fd, state->original_mode.value());
+            return rw_result_t(rw_result_t::ERRNO_ERROR_FILE_UNTOUCHED, saved_errno);
+          }
+          struct stat file_info;
+          if (fstat(state->fd, &file_info) == 0 && (file_info.st_dev != state->readonly_dev ||
+                                                    file_info.st_ino != state->readonly_ino)) {
+            return rw_result_t(rw_result_t::RACE_ON_FILE);
+          }
+          close(state->readonly_fd);
+          state->readonly_fd = -1;
+      }
+      // FALLTHROUGH
+    case save_as_process_t::CREATE_BACKUP: {
+      // If the creation of the backup file fails, the user either aborts or allows continuation
+      // without completing the backup. Thus the next state is always WRITING.
+      state->state = save_as_process_t::WRITING;
+      std::string temp_name_str = state->real_name;
+
+      if (option.make_backup) {
+        temp_name_str += "~";
+        if ((state->backup_fd = open(temp_name_str.c_str(), O_CREAT | O_TRUNC | O_WRONLY, 0600)) <
+            0) {
+          return rw_result_t(rw_result_t::BACKUP_FAILED, errno);
         }
       } else {
-        state->state = save_as_process_t::ALLOW_OVERWRITE;
-        if (!state->name.empty()) {
-          return rw_result_t(rw_result_t::FILE_EXISTS);
-        }
-        /* Note that we have a new case in the middle of the else statement
-           here. It is an ugly hack, but it does prevent a lot of code
-           duplication and other hacks. */
-        case save_as_process_t::ALLOW_OVERWRITE:
-          state->state = save_as_process_t::ALLOW_OVERWRITE_READONLY;
-          if (access(state->save_name, W_OK) != 0) {
-            return rw_result_t(rw_result_t::FILE_EXISTS_READONLY);
-          }
-        case save_as_process_t::ALLOW_OVERWRITE_READONLY:
-          std::string temp_name_str = state->real_name;
-
-          if ((idx = temp_name_str.rfind('/')) == std::string::npos) {
-            idx = 0;
-          } else {
-            idx++;
-          }
-
-          temp_name_str.erase(idx);
-          try {
-            temp_name_str.append(".tildeXXXXXX");
-          } catch (std::bad_alloc &ba) {
-            return rw_result_t(rw_result_t::ERRNO_ERROR, ENOMEM);
-          }
-
-          /* Unfortunately, we can't pass the c_str result to mkstemp as we are not allowed
-             to change that string. So we'll just have to copy it into a vector :-( */
-          std::vector<char> temp_name(temp_name_str.begin(), temp_name_str.end());
-          // Ensure nul termination.
-          temp_name.push_back(0);
-          /* Attempt to create a temporary file. If this fails, just write the file
-             directly. The latter has some risk (e.g. file truncation due to full disk,
-             or corruption due to computer crashes), but these are so small that it is
-             worth permitting this if we can't create the temporary file. */
-          if (geteuid() == state->file_info.st_uid &&
-              (state->fd = mkstemp(temp_name.data())) >= 0) {
-            state->temp_name = temp_name.data();
-// Preserve ownership and attributes
-#ifdef HAS_LIBATTR
-            attr_copy_file(state->real_name.c_str(), state->temp_name.c_str(), nullptr, nullptr);
-#endif
-#ifdef HAS_LIBACL
-            perm_copy_file(state->real_name.c_str(), state->temp_name.c_str(), nullptr);
-#endif
-            fchmod(state->fd, state->file_info.st_mode);
-            fchown(state->fd, -1, state->file_info.st_gid);
-            fchown(state->fd, state->file_info.st_uid, -1);
-          } else {
-            state->temp_name.clear();
-            if ((state->fd = open(state->real_name.c_str(), O_WRONLY | O_CREAT, CREATE_MODE)) < 0) {
-              return rw_result_t(rw_result_t::ERRNO_ERROR, errno);
-            }
-          }
-      }
-
-      {
-        transcript_t *handle;
-        transcript_error_t error;
-        if (!state->encoding.empty()) {
-          if ((handle = transcript_open_converter(state->encoding.c_str(), TRANSCRIPT_UTF8, 0,
-                                                  &error)) == nullptr) {
-            return rw_result_t(rw_result_t::CONVERSION_OPEN_ERROR);
-          }
-          encoding = state->encoding;
-        } else if (encoding != "UTF-8") {
-          if ((handle = transcript_open_converter(encoding.c_str(), TRANSCRIPT_UTF8, 0, &error)) ==
-              nullptr) {
-            return rw_result_t(rw_result_t::CONVERSION_OPEN_ERROR);
-          }
+        if ((idx = temp_name_str.rfind('/')) == std::string::npos) {
+          idx = 0;
         } else {
-          handle = nullptr;
+          idx++;
+        }
+
+        temp_name_str.erase(idx);
+        temp_name_str.append("tilde-backup-XXXXXX");
+
+        /* Unfortunately, we can't pass the c_str result to mkstemp as we are not allowed to change
+           that string. So we'll just have to copy it into a vector :-( */
+        std::vector<char> temp_name(temp_name_str.begin(), temp_name_str.end());
+        // Ensure nul termination.
+        temp_name.push_back(0);
+        if ((state->backup_fd = mkstemp(temp_name.data())) >= 0) {
+          state->temp_name = temp_name.data();
+        } else {
+          return rw_result_t(errno == ENOSPC ? rw_result_t::ERRNO_ERROR_FILE_UNTOUCHED
+                                             : rw_result_t::BACKUP_FAILED,
+                             errno);
         }
-        // FIXME: if the new fails, the handle will remain open!
-        // FIXME: if the new fails, this will abort with an exception! This should not happen!
-        state->wrapper = new file_write_wrapper_t(state->fd, handle);
-        state->i = 0;
-        state->state = save_as_process_t::WRITING;
       }
+      int error = copy_file(state->fd, state->backup_fd);
+      if (error != 0) {
+        return rw_result_t(
+            errno == ENOSPC ? rw_result_t::ERRNO_ERROR_FILE_UNTOUCHED : rw_result_t::BACKUP_FAILED,
+            error);
+      }
+      if (fsync(state->backup_fd) < 0 || close(state->backup_fd) < 0) {
+        return rw_result_t(
+            errno == ENOSPC ? rw_result_t::ERRNO_ERROR_FILE_UNTOUCHED : rw_result_t::BACKUP_FAILED,
+            errno);
+      }
+      state->backup_saved = true;
+      state->backup_fd = -1;
+    }
+      // FALLTHROUGH
     case save_as_process_t::WRITING: {
-      if (strip_spaces.is_valid() ? strip_spaces.value() : option.strip_spaces) {
-        do_strip_spaces();
+#ifdef HAS_POSIX_FALLOCATE
+      // Use posix_fallocate to attempt to pre-allocate the required size of the file. If the call
+      // fails with ENOSPC or EFBIG, stop writing and report an error to the user. All other error
+      // codes are ignored.
+      if (posix_fallocate(state->fd, 0, state->computed_length) < 0 &&
+          (errno == ENOSPC || errno == EFBIG)) {
+        // We want the backup to be removed (if it exists), and we didn't change anything, so we
+        // close the file here and set the fd to -1.
+        close(state->fd);
+        state->fd = -1;
+        return rw_result_t(rw_result_t::ERRNO_ERROR_FILE_UNTOUCHED);
+      }
+#endif
+      int conversion_flags = state->wrapper->conversion_flags();
+      state->wrapper =
+          t3widget::make_unique<file_write_wrapper_t>(state->fd, state->conversion_handle);
+      state->wrapper->add_conversion_flags(conversion_flags);
+      state->i = 0;
+      if (lseek(state->fd, 0, SEEK_SET) < 0) {
+        return rw_result_t(rw_result_t::ERRNO_ERROR);
       }
       try {
         for (; state->i < size(); state->i++) {
@@ -344,30 +417,39 @@ rw_result_t file_buffer_t::save(save_as_process_t *state) {
           state->wrapper->write(data.data(), data.size());
         }
       } catch (rw_result_t error) {
+        // Don't attempt to retry imprecise conversions, as they should have been caught earlier.
+        // Also, restarting the conversion may append the current line to an already partially
+        // written line.
+        if (error == rw_result_t::CONVERSION_IMPRECISE) {
+          return rw_result_t(rw_result_t::CONVERSION_ERROR);
+        }
         return error;
       }
 
-      // If the file is being overwritten, truncate it to the written size.
-      if (state->temp_name.empty()) {
-        off_t curr_pos = lseek(state->fd, 0, SEEK_CUR);
-        if (curr_pos >= 0) {
-          ftruncate(state->fd, curr_pos);
-        }
+      // Truncate it to the written size.
+      int result;
+      while ((result = ftruncate(state->fd, state->wrapper->written_size())) < 0 &&
+             errno == EINTR) {
+      }
+      if (result < 0) {
+        return rw_result_t(rw_result_t::ERRNO_ERROR);
+      }
+      if (fsync(state->fd) < 0) {
+        return rw_result_t(rw_result_t::ERRNO_ERROR);
+      }
+      /* Perform fchmod instead of chmod on the file name, to ensure that we actually change the
+         mode on the file we are interested in. However, we only want to report a problem after
+         cleaning up the rest, as it is more of an advisory nature. */
+      int fchmod_errno = 0;
+      if (state->original_mode.is_valid() && fchmod(state->fd, state->original_mode.value()) < 0) {
+        fchmod_errno = errno;
+      }
+      state->original_mode.reset();
+
+      if (close(state->fd) < 0) {
+        return rw_result_t(rw_result_t::ERRNO_ERROR);
       }
-      fsync(state->fd);
-      close(state->fd);
       state->fd = -1;
-      if (!state->temp_name.empty()) {
-        if (option.make_backup) {
-          std::string backup_name = state->real_name;
-          backup_name += '~';
-          unlink(backup_name.c_str());
-          link(state->real_name.c_str(), backup_name.c_str());
-        }
-        if (rename(state->temp_name.c_str(), state->real_name.c_str()) < 0) {
-          return rw_result_t(rw_result_t::ERRNO_ERROR, errno);
-        }
-      }
 
       if (!state->name.empty()) {
         name = state->name;
@@ -375,10 +457,13 @@ rw_result_t file_buffer_t::save(save_as_process_t *state) {
         name_line.set_text(converted_name);
       }
       set_undo_mark();
+      if (fchmod_errno != 0) {
+        return rw_result_t(rw_result_t::MODE_RESET_FAILED, fchmod_errno);
+      }
       break;
     }
     default:
-      PANIC();
+      return rw_result_t(rw_result_t::INTERNAL_ERROR);
   }
   return rw_result_t(rw_result_t::SUCCESS);
 }
@@ -389,8 +474,8 @@ const char *file_buffer_t::get_encoding() const { return encoding.c_str(); }
 
 text_line_t *file_buffer_t::get_name_line() { return &name_line; }
 
-const edit_window_t::view_parameters_t *file_buffer_t::get_view_parameters() const {
-  return view_parameters.get();
+const edit_window_t::behavior_parameters_t *file_buffer_t::get_behavior_parameters() const {
+  return behavior_parameters.get();
 }
 
 void file_buffer_t::prepare_paint_line(text_pos_t line) {
@@ -751,8 +836,9 @@ void file_buffer_t::toggle_line_comment() {
       }
       set_cursor_pos(new_pos);
     } else {
-      // FIXME: this causes the cursor position to be recorded incorrectly in the undo information.
-      // although one could argue this is to some extent better as it shows the actual edit.
+      // FIXME: this causes the cursor position to be recorded incorrectly in the undo
+      // information. although one could argue this is to some extent better as it shows the
+      // actual edit.
       set_cursor_pos(0);
       insert_block(line_comment);
       set_cursor_pos(saved_cursor.pos + line_comment.size());
@@ -817,10 +903,14 @@ const char *file_buffer_t::get_char_under_cursor(size_t *size) const {
     *size = 1;
     return "\n";
   }
-  int end = line.adjust_position(cursor.pos, 1);
+  text_pos_t end = line.adjust_position(cursor.pos, 1);
   if (cursor.pos == end) {
     return nullptr;
   }
   *size = end - cursor.pos;
   return line.get_data().data() + cursor.pos;
 }
+
+void file_buffer_t::set_top_left_in_behavior_parameters(text_coordinate_t pos) {
+  behavior_parameters->set_top_left(pos);
+}
