@@ -26,16 +26,14 @@ class file_line_t : public text_line_t {
 
  public:
   file_line_t(int buffersize = BUFFERSIZE, file_line_factory_t *_factory = nullptr);
-  file_line_t(const char *_buffer, file_line_factory_t *_factory = nullptr);
-  file_line_t(const char *_buffer, int length, file_line_factory_t *_factory = nullptr);
-  file_line_t(const std::string *str, file_line_factory_t *_factory = nullptr);
+  file_line_t(string_view _buffer, file_line_factory_t *_factory = nullptr);
 
   void set_highlight_start(int state);
   int get_highlight_end();
-  int get_highlight_idx(int i);
+  int get_highlight_idx(text_pos_t i) const;
 
  protected:
-  t3_attr_t get_base_attr(int i, const paint_info_t *info) override;
+  t3_attr_t get_base_attr(text_pos_t i, const paint_info_t &info) const override;
 };
 
 class file_line_factory_t : public text_line_factory_t {
@@ -44,10 +42,8 @@ class file_line_factory_t : public text_line_factory_t {
 
  public:
   file_line_factory_t(file_buffer_t *_file_buffer);
-  text_line_t *new_text_line_t(int buffersize = BUFFERSIZE) override;
-  text_line_t *new_text_line_t(const char *_buffer) override;
-  text_line_t *new_text_line_t(const char *_buffer, int length) override;
-  text_line_t *new_text_line_t(const std::string *str) override;
+  std::unique_ptr<text_line_t> new_text_line_t(int buffersize = BUFFERSIZE) override;
+  std::unique_ptr<text_line_t> new_text_line_t(string_view _buffer) override;
   file_buffer_t *get_file_buffer() const;
 };
 
