@@ -15,6 +15,7 @@
 
 #include "tilde/filebuffer.h"
 #include "tilde/filestate.h"
+#include "tilde/log.h"
 #include "tilde/main.h"
 #include "tilde/openfiles.h"
 #include "tilde/option.h"
@@ -27,7 +28,9 @@ load_process_t::load_process_t(const callback_t &cb)
       wrapper(nullptr),
       encoding("UTF-8"),
       fd(-1),
-      buffer_used(true) {}
+      buffer_used(true) {
+  set_up_connections();
+}
 
 load_process_t::load_process_t(const callback_t &cb, const char *name, const char *_encoding,
                                bool missing_ok)
@@ -38,7 +41,26 @@ load_process_t::load_process_t(const callback_t &cb, const char *name, const cha
       wrapper(nullptr),
       encoding(_encoding == nullptr ? "UTF-8" : _encoding),
       fd(-1),
-      buffer_used(true) {}
+      buffer_used(true) {
+  set_up_connections();
+}
+
+void load_process_t::set_up_connections() {
+  connections.push_back(continue_abort_dialog->connect_activate([this] { run(); }, 0));
+  connections.push_back(continue_abort_dialog->connect_activate([this] { abort(); }, 1));
+  connections.push_back(continue_abort_dialog->connect_closed([this] { abort(); }));
+
+  connections.push_back(preserve_bom_dialog->connect_activate([this] { preserve_bom(); }, 0));
+  connections.push_back(preserve_bom_dialog->connect_activate([this] { remove_bom(); }, 1));
+  connections.push_back(preserve_bom_dialog->connect_closed([this] { preserve_bom(); }));
+
+  connections.push_back(
+      open_file_dialog->connect_file_selected(bind_front(&load_process_t::file_selected, this)));
+  connections.push_back(open_file_dialog->connect_closed([this] { abort(); }));
+
+  connections.push_back(
+      encoding_dialog->connect_activate(bind_front(&load_process_t::encoding_selected, this)));
+}
 
 void load_process_t::abort() {
   delete file;
@@ -51,18 +73,12 @@ bool load_process_t::step() {
   rw_result_t rw_result;
 
   if (state == SELECT_FILE) {
-    connections.push_back(
-        open_file_dialog->connect_file_selected(bind_front(&load_process_t::file_selected, this)));
-    connections.push_back(open_file_dialog->connect_closed([this] { abort(); }));
     open_file_dialog->reset();
     open_file_dialog->show();
-    connections.push_back(
-        encoding_dialog->connect_activate(bind_front(&load_process_t::encoding_selected, this)));
     encoding_dialog->set_encoding(encoding.c_str());
     return false;
   }
 
-  disconnect();
   switch ((rw_result = file->load(this))) {
     case rw_result_t::SUCCESS:
       result = true;
@@ -92,18 +108,12 @@ bool load_process_t::step() {
       break;
     case rw_result_t::CONVERSION_IMPRECISE:
       printf_into(&message, "Conversion from encoding %s is irreversible", file->get_encoding());
-      connections.push_back(continue_abort_dialog->connect_activate([this] { run(); }, 0));
-      connections.push_back(continue_abort_dialog->connect_activate([this] { abort(); }, 1));
-      connections.push_back(continue_abort_dialog->connect_closed([this] { abort(); }));
       continue_abort_dialog->set_message(message);
       continue_abort_dialog->show();
       return false;
     case rw_result_t::CONVERSION_ILLEGAL:
       printf_into(&message, "Conversion from encoding %s encountered illegal characters",
                   file->get_encoding());
-      connections.push_back(continue_abort_dialog->connect_activate([this] { run(); }, 0));
-      connections.push_back(continue_abort_dialog->connect_activate([this] { abort(); }, 1));
-      connections.push_back(continue_abort_dialog->connect_closed([this] { abort(); }));
       continue_abort_dialog->set_message(message);
       continue_abort_dialog->show();
       return false;
@@ -114,14 +124,20 @@ bool load_process_t::step() {
       error_dialog->show();
       break;
     case rw_result_t::BOM_FOUND:
-      connections.push_back(preserve_bom_dialog->connect_activate([this] { preserve_bom(); }, 0));
-      connections.push_back(preserve_bom_dialog->connect_activate([this] { remove_bom(); }, 1));
-      connections.push_back(preserve_bom_dialog->connect_closed([this] { preserve_bom(); }));
       preserve_bom_dialog->show();
       return false;
     default:
       PANIC();
   }
+  auto recent_files_iter = recent_files.find(open_files.back()->get_name());
+  if (recent_files_iter != recent_files.end()) {
+    if (option.restore_cursor_position) {
+      text_coordinate_t position = (*recent_files_iter)->get_position();
+      open_files.back()->goto_pos(position.line + 1, position.pos + 1);
+      open_files.back()->set_top_left_in_behavior_parameters((*recent_files_iter)->get_top_left());
+    }
+    recent_files.erase(recent_files_iter);
+  }
   return true;
 }
 
@@ -129,7 +145,7 @@ void load_process_t::file_selected(const std::string &name) {
   open_files_t::iterator iter;
   if ((iter = open_files.contains(name.c_str())) != open_files.end()) {
     file = *iter;
-    done(true);
+    done();
     return;
   }
 
@@ -147,7 +163,7 @@ file_buffer_t *load_process_t::get_file_buffer() {
   return nullptr;
 }
 
-load_process_t::~load_process_t() {
+void load_process_t::cleanup() {
 #ifdef DEBUG
   ASSERT(result || file == nullptr);
 #endif
@@ -180,54 +196,74 @@ save_as_process_t::save_as_process_t(const callback_t &cb, file_buffer_t *_file,
     : stepped_process_t(cb),
       state(SELECT_FILE),
       file(_file),
-      allow_highlight_change(_allow_highlight_change),
-      highlight_changed(false),
-      save_name(nullptr),
-      fd(-1),
-      wrapper(nullptr) {}
+      allow_highlight_change(_allow_highlight_change) {
+  connections.push_back(continue_abort_dialog->connect_activate([this] { run(); }, 0));
+  connections.push_back(continue_abort_dialog->connect_activate([this] { abort(); }, 1));
+  connections.push_back(continue_abort_dialog->connect_closed([this] { abort(); }));
+
+  connections.push_back(
+      save_as_dialog->connect_file_selected(bind_front(&save_as_process_t::file_selected, this)));
+  connections.push_back(save_as_dialog->connect_closed([this] { abort(); }));
+
+  connections.push_back(
+      encoding_dialog->connect_activate(bind_front(&save_as_process_t::encoding_selected, this)));
+}
 
 bool save_as_process_t::step() {
   std::string message;
   rw_result_t rw_result;
 
   if (state == SELECT_FILE) {
-    connections.push_back(
-        save_as_dialog->connect_file_selected(bind_front(&save_as_process_t::file_selected, this)));
-    connections.push_back(save_as_dialog->connect_closed([this] { abort(); }));
-
     save_as_dialog->set_from_file(file->get_name());
     save_as_dialog->show();
-    connections.push_back(
-        encoding_dialog->connect_activate(bind_front(&save_as_process_t::encoding_selected, this)));
     encoding_dialog->set_encoding(file->get_encoding());
     return false;
   }
 
-  disconnect();
   switch ((rw_result = file->save(this))) {
     case rw_result_t::SUCCESS:
       result = true;
       break;
     case rw_result_t::FILE_EXISTS:
       printf_into(&message, "File '%s' already exists", name.c_str());
-      connections.push_back(continue_abort_dialog->connect_activate([this] { run(); }, 0));
-      connections.push_back(continue_abort_dialog->connect_activate([this] { abort(); }, 1));
-      connections.push_back(continue_abort_dialog->connect_closed([this] { abort(); }));
       continue_abort_dialog->set_message(message);
       continue_abort_dialog->show();
       return false;
-    case rw_result_t::FILE_EXISTS_READONLY:
-      printf_into(&message, "File '%s' is readonly", save_name);
-      connections.push_back(continue_abort_dialog->connect_activate([this] { run(); }, 0));
-      connections.push_back(continue_abort_dialog->connect_activate([this] { abort(); }, 1));
-      connections.push_back(continue_abort_dialog->connect_closed([this] { abort(); }));
+    case rw_result_t::BACKUP_FAILED:
+      printf_into(
+          &message,
+          "Could not create a backup file: %s.\n\nThis could result in data loss if Tilde is "
+          "unable to complete writing the file",
+          strerror(rw_result.get_errno_error()));
       continue_abort_dialog->set_message(message);
       continue_abort_dialog->show();
       return false;
     case rw_result_t::ERRNO_ERROR:
-      printf_into(&message, "Could not save file: %s", strerror(rw_result.get_errno_error()));
+    case rw_result_t::ERRNO_ERROR_FILE_UNTOUCHED:
+      printf_into(&message, "Could not save file: %s.", strerror(rw_result.get_errno_error()));
+      if (rw_result == rw_result_t::ERRNO_ERROR_FILE_UNTOUCHED) {
+        message.append(
+            "\n\nThe original file has not been touched. Save the current buffer to another "
+            "location to ensure its contents are preserved!");
+      } else if (backup_saved) {
+        message.append("\n\nThe original contents of the file can still be retrieved from ");
+        // FIXME: the file names probably needs to be converted from some other character set.
+        if (!temp_name.empty()) {
+          message.append(temp_name);
+        } else {
+          message.append(name);
+          message.append("~");
+        }
+        message.append(
+            ".  Save the current buffer to another location to ensure its contents are preserved!");
+      } else {
+        message.append(
+            "\n\nThe original file contents are lost. Save the current buffer to some other "
+            "location to ensure its contents are preserved!");
+      }
       error_dialog->set_message(message);
       error_dialog->show();
+      abort();
       break;
     case rw_result_t::CONVERSION_ERROR:
       if (encoding.empty()) {
@@ -237,21 +273,50 @@ bool save_as_process_t::step() {
                   transcript_strerror(rw_result.get_transcript_error()));
       error_dialog->set_message(message);
       error_dialog->show();
+      abort();
       break;
     case rw_result_t::CONVERSION_IMPRECISE:
       if (encoding.empty()) {
         encoding = file->get_encoding();
       }
       i++;
-      printf_into(&message, "Conversion into encoding %s is irreversible", encoding.c_str());
-      connections.push_back(continue_abort_dialog->connect_activate([this] { run(); }, 0));
-      connections.push_back(continue_abort_dialog->connect_activate([this] { abort(); }, 1));
-      connections.push_back(continue_abort_dialog->connect_closed([this] { abort(); }));
+      printf_into(&message,
+                  "Conversion into encoding %s is irreversible\n\nThe loaded buffer will continue "
+                  "to hold the original text, but the on-disk version will differ.",
+                  encoding.c_str());
       continue_abort_dialog->set_message(message);
       continue_abort_dialog->show();
       return false;
+    case rw_result_t::READ_ONLY_FILE:
+      printf_into(&message, "File %s is read-only", save_name);
+      continue_abort_dialog->set_message(message);
+      continue_abort_dialog->show();
+      return false;
+    case rw_result_t::MODE_RESET_FAILED:
+      printf_into(
+          &message,
+          "The file %s was written successfully, but changing back the mode bits on the file "
+          "failed: %s",
+          name.c_str(), strerror(rw_result.get_errno_error()));
+      error_dialog->set_message(message);
+      error_dialog->show();
+      abort();
+      break;
+    case rw_result_t::RACE_ON_FILE:
+      printf_into(&message,
+                  "Opening file '%s' after changing the mode opened a different file. The file was "
+                  "not written.",
+                  name.c_str());
+      abort();
+      break;
     default:
-      PANIC();
+      printf_into(&message,
+                  "An unknown error occurred during saving. The file has not been saved and may be "
+                  "damaged!");
+      error_dialog->set_message(message);
+      error_dialog->show();
+      abort();
+      break;
   }
   return true;
 }
@@ -269,16 +334,29 @@ void save_as_process_t::file_selected(const std::string &_name) {
 
 void save_as_process_t::encoding_selected(const std::string *_encoding) { encoding = *_encoding; }
 
-save_as_process_t::~save_as_process_t() {
+void save_as_process_t::cleanup() {
+  if (backup_fd >= 0) {
+    close(backup_fd);
+  }
+  if (readonly_fd >= 0) {
+    if (original_mode.is_valid()) {
+      fchmod(fd, original_mode.value());
+      original_mode.reset();
+    }
+    close(readonly_fd);
+  }
   if (fd >= 0) {
+    if (original_mode.is_valid()) {
+      fchmod(fd, original_mode.value());
+    }
     close(fd);
-    if (temp_name.empty()) {
-      unlink(name.c_str());
-    } else {
-      unlink(temp_name.c_str());
-    }
+  } else if (!temp_name.empty()) {
+    // Remove the backup file (not the ~ backup, but the temporary file we may have created).
+    unlink(temp_name.c_str());
+  }
+  if (conversion_handle) {
+    transcript_close_converter(conversion_handle);
   }
-  delete wrapper;
 }
 
 void save_as_process_t::execute(const callback_t &cb, file_buffer_t *_file) {
@@ -304,6 +382,10 @@ close_process_t::close_process_t(const callback_t &cb, file_buffer_t *_file)
   if (!_file->is_modified()) {
     state = CLOSE;
   }
+  connections.push_back(close_confirm_dialog->connect_activate([this] { do_save(); }, 0));
+  connections.push_back(close_confirm_dialog->connect_activate([this] { dont_save(); }, 1));
+  connections.push_back(close_confirm_dialog->connect_activate([this] { abort(); }, 2));
+  connections.push_back(close_confirm_dialog->connect_closed([this] { abort(); }));
 }
 
 bool close_process_t::step() {
@@ -318,7 +400,6 @@ bool close_process_t::step() {
     }
   }
 
-  disconnect();
   if (state == CLOSE) {
     recent_files.push_front(file);
     /* Can't delete the file_buffer_t here, because on switching buffers the
@@ -331,10 +412,6 @@ bool close_process_t::step() {
     std::string message;
     printf_into(&message, "Save changes to '%s'",
                 file->get_name().empty() ? "(Untitled)" : file->get_name().c_str());
-    connections.push_back(close_confirm_dialog->connect_activate([this] { do_save(); }, 0));
-    connections.push_back(close_confirm_dialog->connect_activate([this] { dont_save(); }, 1));
-    connections.push_back(close_confirm_dialog->connect_activate([this] { abort(); }, 2));
-    connections.push_back(close_confirm_dialog->connect_closed([this] { abort(); }));
     close_confirm_dialog->set_message(message);
     close_confirm_dialog->show();
   } else {
@@ -360,7 +437,12 @@ void close_process_t::execute(const callback_t &cb, file_buffer_t *_file) {
 const file_buffer_t *close_process_t::get_file_buffer_ptr() { return file; }
 
 exit_process_t::exit_process_t(const callback_t &cb)
-    : stepped_process_t(cb), iter(open_files.begin()) {}
+    : stepped_process_t(cb), iter(open_files.begin()) {
+  connections.push_back(close_confirm_dialog->connect_activate([this] { do_save(); }, 0));
+  connections.push_back(close_confirm_dialog->connect_activate([this] { dont_save(); }, 1));
+  connections.push_back(close_confirm_dialog->connect_activate([this] { abort(); }, 2));
+  connections.push_back(close_confirm_dialog->connect_closed([this] { abort(); }));
+}
 
 bool exit_process_t::step() {
   for (; iter != open_files.end(); iter++) {
@@ -368,19 +450,15 @@ bool exit_process_t::step() {
       std::string message;
       printf_into(&message, "Save changes to '%s'",
                   (*iter)->get_name().empty() ? "(Untitled)" : (*iter)->get_name().c_str());
-      connections.push_back(close_confirm_dialog->connect_activate([this] { do_save(); }, 0));
-      connections.push_back(close_confirm_dialog->connect_activate([this] { dont_save(); }, 1));
-      connections.push_back(close_confirm_dialog->connect_activate([this] { abort(); }, 2));
-      connections.push_back(close_confirm_dialog->connect_closed([this] { abort(); }));
       close_confirm_dialog->set_message(message);
       close_confirm_dialog->show();
       return false;
     }
   }
-  delete this;
-  exit_main_loop(EXIT_SUCCESS);
-  /*	in_step = false;
-          return true;*/
+  for (file_buffer_t *buffer : open_files) {
+    recent_files.push_front(buffer);
+  }
+  return true;
 }
 
 void exit_process_t::do_save() {
@@ -388,28 +466,37 @@ void exit_process_t::do_save() {
 }
 
 void exit_process_t::dont_save() {
-  iter++;
+  ++iter;
   run();
 }
 
 void exit_process_t::save_done(stepped_process_t *process) {
   if (process->get_result()) {
-    iter++;
+    ++iter;
     run();
   } else {
     abort();
   }
 }
 
-void exit_process_t::execute(const callback_t &cb) { (new exit_process_t(cb))->run(); }
+void exit_process_t::execute(const callback_t &cb) {
+  (new exit_process_t([cb](stepped_process_t *process) {
+    lprintf("Exit process callback with result %d\n", process->get_result());
+    cb(process);
+    if (process->get_result()) {
+      exit_main_loop(EXIT_SUCCESS);
+    }
+  }))->run();
+}
 
-open_recent_process_t::open_recent_process_t(const callback_t &cb) : load_process_t(cb) {}
+open_recent_process_t::open_recent_process_t(const callback_t &cb) : load_process_t(cb) {
+  connections.push_back(open_recent_dialog->connect_file_selected(
+      bind_front(&open_recent_process_t::recent_file_selected, this)));
+  connections.push_back(open_recent_dialog->connect_closed([this] { abort(); }));
+}
 
 bool open_recent_process_t::step() {
   if (state == SELECT_FILE) {
-    connections.push_back(open_recent_dialog->connect_file_selected(
-        bind_front(&open_recent_process_t::recent_file_selected, this)));
-    connections.push_back(open_recent_dialog->connect_closed([this] { abort(); }));
     open_recent_dialog->show();
     return false;
   }
@@ -423,7 +510,7 @@ void open_recent_process_t::recent_file_selected(recent_file_info_t *_info) {
   run();
 }
 
-open_recent_process_t::~open_recent_process_t() {
+void open_recent_process_t::cleanup() {
   if (result) {
     recent_files.erase(info);
   }
@@ -437,7 +524,6 @@ load_cli_file_process_t::load_cli_file_process_t(const callback_t &cb)
     : stepped_process_t(cb),
       iter(cli_option.files.begin()),
       in_load(false),
-      in_step(false),
       encoding_selected(false) {}
 
 bool load_cli_file_process_t::step() {
@@ -457,7 +543,6 @@ bool load_cli_file_process_t::step() {
     }
   }
 
-  in_step = true;
   while (iter != cli_option.files.end()) {
     text_pos_t line = -1, pos = -1;
     std::string filename = *iter;
@@ -470,7 +555,6 @@ bool load_cli_file_process_t::step() {
     load_process_t::execute(bind_front(&load_cli_file_process_t::load_done, this), filename.c_str(),
                             encoding.c_str(), true);
     if (in_load) {
-      in_step = false;
       return false;
     }
     open_files.back()->goto_pos(line, pos);
@@ -483,7 +567,7 @@ void load_cli_file_process_t::load_done(stepped_process_t *process) {
   (void)process;
 
   in_load = false;
-  iter++;
+  ++iter;
   if (!in_step) {
     run();
   }
