@@ -14,7 +14,9 @@
 
 #include <csignal>
 #include <cstdlib>
+#include <list>
 #include <memory.h>
+#include <set>
 #include <t3widget/key_binding.h>
 #include <t3widget/widget.h>
 
@@ -31,6 +33,7 @@
 #include "tilde/log.h"
 #include "tilde/openfiles.h"
 #include "tilde/option.h"
+#include "tilde/option_access.h"
 #include "tilde/string_util.h"
 
 using namespace t3widget;
@@ -63,17 +66,17 @@ class main_t : public main_window_base_t {
   menu_bar_t *menu;
   menu_panel_t *panel;
   split_t *split;
+  std::set<file_edit_window_t *> edit_windows;
 
-  select_buffer_dialog_t *select_buffer_dialog;
-  message_dialog_t *about_dialog;
-  buffer_options_dialog_t *buffer_options_dialog, *default_options_dialog;
-  misc_options_dialog_t *misc_options_dialog;
-  highlight_dialog_t *highlight_dialog;
-  attributes_dialog_t *attributes_dialog;
+  std::unique_ptr<select_buffer_dialog_t> select_buffer_dialog;
+  std::unique_ptr<message_dialog_t> about_dialog;
+  std::unique_ptr<buffer_options_dialog_t> buffer_options_dialog, default_options_dialog;
+  std::unique_ptr<misc_options_dialog_t> misc_options_dialog;
+  std::unique_ptr<highlight_dialog_t> highlight_dialog;
+  std::unique_ptr<attributes_dialog_t> attributes_dialog;
 
  public:
   main_t();
-  ~main_t() override;
   bool process_key(t3widget::key_t key) override;
   bool set_size(optint height, optint width) override;
   void load_cli_files_done(stepped_process_t *process);
@@ -89,7 +92,6 @@ class main_t : public main_window_base_t {
   void set_buffer_options();
   void set_default_options();
   void set_interface_options();
-  void set_default_interface_options();
   void set_misc_options();
   void set_highlight(t3_highlight_t *highlight, const char *name);
   void save_as_done(stepped_process_t *process);
@@ -179,6 +181,8 @@ main_t::main_t() {
   panel->insert_item(nullptr, "_Current Buffer...", "", action_id_t::OPTIONS_BUFFER);
   panel->insert_item(nullptr, "Buffer _Defaults...", "", action_id_t::OPTIONS_DEFAULTS);
   panel->insert_item(nullptr, "_Interface...", "", action_id_t::OPTIONS_INTERFACE);
+  panel->insert_item(nullptr, "Inter_face defaults...", "",
+                     action_id_t::OPTIONS_INTERFACE_DEFAULTS);
 
   panel->insert_item(nullptr, "_Miscellaneous...", "", action_id_t::OPTIONS_MISC);
 
@@ -187,11 +191,15 @@ main_t::main_t() {
   //~ panel->insert_item(nullptr, "_Help", "F1", action_id_t::HELP_HELP);
   panel->insert_item(nullptr, "_About", "", action_id_t::HELP_ABOUT);
 
-  split = emplace_back<split_t>(make_unique<file_edit_window_t>());
+  {
+    std::unique_ptr<file_edit_window_t> new_window = make_unique<file_edit_window_t>();
+    edit_windows.insert(new_window.get());
+    split = emplace_back<split_t>(std::move(new_window));
+  }
   split->set_position(!option.hide_menubar, 0);
   split->set_size(window.get_height() - !option.hide_menubar, window.get_width());
 
-  select_buffer_dialog = new select_buffer_dialog_t(11, window.get_width() - 4);
+  select_buffer_dialog = make_unique<select_buffer_dialog_t>(11, window.get_width() - 4);
   select_buffer_dialog->center_over(this);
   select_buffer_dialog->connect_activate(bind_front(&main_t::switch_buffer, this));
 
@@ -225,31 +233,32 @@ main_t::main_t() {
   open_recent_dialog = new open_recent_dialog_t(11, window.get_width() - 4);
   open_recent_dialog->center_over(this);
 
-  about_dialog = new message_dialog_t(45, "About", {"Close"});
+  about_dialog = t3widget::make_unique<message_dialog_t>(
+      45, std::string("About"), std::initializer_list<string_view>{"Close"});
   about_dialog->center_over(this);
   about_dialog->set_max_text_height(13);
   about_dialog->set_message(
       // clang-format off
-      "Tilde - The intuitive text editor\n\nVersion 1.0.0\n"
+      "Tilde - The intuitive text editor\n\nVersion 1.1.2\n"
       "Copyright (c) 2011-2018 G.P. Halkes\n\n"  // @copyright
       "The Tilde text editor is licensed under the GNU General Public License version 3. "
       "You should have received a copy of the GNU General Public License along with this program. "
       "If not, see <http://www.gnu.org/licenses/>.");
   // clang-format on
 
-  buffer_options_dialog = new buffer_options_dialog_t("Current Buffer");
+  buffer_options_dialog = make_unique<buffer_options_dialog_t>("Current Buffer");
   buffer_options_dialog->center_over(this);
   buffer_options_dialog->connect_activate([this] { set_buffer_options(); });
 
-  default_options_dialog = new buffer_options_dialog_t("Buffer Defaults");
+  default_options_dialog = make_unique<buffer_options_dialog_t>("Buffer Defaults");
   default_options_dialog->center_over(this);
   default_options_dialog->connect_activate([this] { set_default_options(); });
 
-  misc_options_dialog = new misc_options_dialog_t("Miscellaneous");
+  misc_options_dialog = make_unique<misc_options_dialog_t>("Miscellaneous");
   misc_options_dialog->center_over(this);
   misc_options_dialog->connect_activate([this] { set_misc_options(); });
 
-  highlight_dialog = new highlight_dialog_t(window.get_height() - 4, 40);
+  highlight_dialog = make_unique<highlight_dialog_t>(window.get_height() - 4, 40);
   highlight_dialog->center_over(this);
   highlight_dialog->connect_language_selected(bind_front(&main_t::set_highlight, this));
 
@@ -263,21 +272,9 @@ main_t::main_t() {
   character_details_dialog = new character_details_dialog_t(8, MESSAGE_DIALOG_WIDTH);
   character_details_dialog->center_over(this);
 
-  attributes_dialog = new attributes_dialog_t(ATTRIBUTES_DIALOG_WIDTH);
+  attributes_dialog = make_unique<attributes_dialog_t>(ATTRIBUTES_DIALOG_WIDTH);
   attributes_dialog->center_over(this);
   attributes_dialog->connect_activate([this] { set_interface_options(); });
-  attributes_dialog->connect_save_defaults([this] { set_default_interface_options(); });
-}
-
-main_t::~main_t() {
-#ifdef TILDE_DEBUG
-  delete select_buffer_dialog;
-  delete about_dialog;
-  delete buffer_options_dialog, delete default_options_dialog;
-  delete attributes_dialog;
-  delete misc_options_dialog;
-  delete highlight_dialog;
-#endif
 }
 
 bool main_t::process_key(t3widget::key_t key) {
@@ -348,6 +345,7 @@ void main_t::menu_activated(int id) {
     }
 
     case action_id_t::FILE_CLOSE:
+      get_current()->save_behavior_parameters_in_buffer();
       close_process_t::execute(bind_front(&main_t::close_cb, this), get_current()->get_text());
       break;
     case action_id_t::FILE_SAVE:
@@ -368,6 +366,10 @@ void main_t::menu_activated(int id) {
       break;
 
     case action_id_t::FILE_EXIT:
+      /* Save the view parameters such that saving of the recent file info works correctly. */
+      for (auto *window : edit_windows) {
+        window->save_behavior_parameters_in_buffer();
+      }
       exit_process_t::execute(stepped_process_t::ignore_result);
       break;
 
@@ -428,12 +430,12 @@ void main_t::menu_activated(int id) {
       break;
 
     case action_id_t::WINDOWS_NEXT_BUFFER: {
-      file_edit_window_t *current = static_cast<file_edit_window_t *>(split->get_current());
+      file_edit_window_t *current = get_current();
       current->set_text(open_files.next_buffer(current->get_text()));
       break;
     }
     case action_id_t::WINDOWS_PREV_BUFFER: {
-      file_edit_window_t *current = static_cast<file_edit_window_t *>(split->get_current());
+      file_edit_window_t *current = get_current();
       current->set_text(open_files.previous_buffer(current->get_text()));
       break;
     }
@@ -443,8 +445,10 @@ void main_t::menu_activated(int id) {
     case action_id_t::WINDOWS_HSPLIT:
     case action_id_t::WINDOWS_VSPLIT: {
       file_buffer_t *new_file = open_files.next_buffer(nullptr);
+      std::unique_ptr<file_edit_window_t> new_window = make_unique<file_edit_window_t>(new_file);
+      edit_windows.insert(new_window.get());
       // If new_file is nullptr, a new file_buffer_t will be created
-      split->split(make_unique<file_edit_window_t>(new_file), id == action_id_t::WINDOWS_HSPLIT);
+      split->split(std::move(new_window), id == action_id_t::WINDOWS_HSPLIT);
       break;
     }
     case action_id_t::WINDOWS_NEXT_WINDOW:
@@ -454,14 +458,16 @@ void main_t::menu_activated(int id) {
       split->previous();
       break;
     case action_id_t::WINDOWS_MERGE: {
-      file_edit_window_t *widget = static_cast<file_edit_window_t *>(split->unsplit().release());
+      std::unique_ptr<file_edit_window_t> widget(
+          static_cast<file_edit_window_t *>(split->unsplit().release()));
       if (widget == nullptr) {
         message_dialog->set_message("Can not close the last window.");
         message_dialog->center_over(this);
         message_dialog->show();
       } else {
         file_buffer_t *text = widget->get_text();
-        delete widget;
+        edit_windows.erase(widget.get());
+        widget.reset();
         if (text->get_name().empty() && !text->is_modified()) {
           delete text;
         }
@@ -496,6 +502,12 @@ void main_t::menu_activated(int id) {
       default_options_dialog->show();
       break;
     case action_id_t::OPTIONS_INTERFACE:
+      attributes_dialog->set_change_defaults(false);
+      attributes_dialog->set_values_from_options();
+      attributes_dialog->show();
+      break;
+    case action_id_t::OPTIONS_INTERFACE_DEFAULTS:
+      attributes_dialog->set_change_defaults(true);
       attributes_dialog->set_values_from_options();
       attributes_dialog->show();
       break;
@@ -575,14 +587,7 @@ void main_t::set_default_options() {
 void main_t::set_interface_options() {
   /* First set color mode, because that resets all the attributes to the defaults. */
   set_color_mode(option.color);
-  attributes_dialog->set_term_options_from_values();
-  write_config();
-}
-
-void main_t::set_default_interface_options() {
-  /* First set color mode, because that resets all the attributes to the defaults. */
-  set_color_mode(option.color);
-  attributes_dialog->set_default_options_from_values();
+  attributes_dialog->set_options_from_values();
   write_config();
 }
 
@@ -872,9 +877,16 @@ int main(int argc, char *argv[]) {
     input_selection_dialog = input_message;
   }
 
+  if (option.save_recent_files) {
+    recent_files.load_from_disk();
+  }
+
   load_cli_file_process_t::execute(bind_front(&main_t::load_cli_files_done, main_window));
   setup_signal_handlers();
   int retval = main_loop();
+  if (option.save_recent_files) {
+    recent_files.write_to_disk();
+  }
 #ifdef TILDE_DEBUG
   delete continue_abort_dialog;
   delete open_file_dialog;
@@ -885,8 +897,11 @@ int main(int argc, char *argv[]) {
   delete encoding_dialog;
   delete main_window;
   delete preserve_bom_dialog;
+  delete character_details_dialog;
   recent_files.cleanup();
-  open_files.cleanup();
+  while (!open_files.empty()) {
+    delete *open_files.begin();
+  }
   cleanup();
   transcript_finalize();
 #endif
