@@ -15,9 +15,11 @@
 #define UTIL_H
 
 #include <climits>
+#include <cstdio>
 #include <cstdlib>
 #include <list>
 #include <string>
+#include <t3config/config.h>
 #include <t3widget/signals.h>
 
 #ifdef __GNUC__
@@ -87,14 +89,15 @@ class stepped_process_t {
   using callback_t = std::function<void(stepped_process_t *)>;
   callback_t done_cb;
   bool result;
+  bool in_step = false;
 
   stepped_process_t();
   explicit stepped_process_t(const callback_t &cb);
   virtual bool step() = 0;
   void run();
   void abort();
-  void disconnect();
-  void done(bool _result);
+  void done();
+  virtual void cleanup() {}
 
  public:
   bool get_result();
@@ -108,10 +111,7 @@ void set_limits();
 std::string canonicalize_path(const char *path);
 void printf_into(std::string *message, const char *format, ...);
 
-#define MAX_HIGHLIGHTS 20
-
 int map_highlight(void *data, const char *name);
-const char *reverse_map_highlight(int idx);
 
 #define ARRAY_SIZE(_x) (sizeof(_x) / sizeof(_x[0]))
 
@@ -124,4 +124,16 @@ const char *reverse_map_highlight(int idx);
     return _name.connect(_slot);                                         \
   }
 
+struct fclose_deleter {
+  void operator()(FILE *file) { fclose(file); }
+};
+
+struct t3_config_deleter {
+  void operator()(t3_config_t *config) { t3_config_delete(config); }
+};
+
+struct t3_schema_deleter {
+  void operator()(t3_config_schema_t *schema) { t3_config_delete_schema(schema); }
+};
+
 #endif
